SUCCESS = (0, '')

ACCESS_ID_NOT_NUMBER = (-800, 'accessId is not number.')
ACCESS_ID_ILLEGAL = (-801, 'accessId is illegal.')

SECRET_KEY_TYPE_ERROR = (-810, 'secretKey is not unicode type.')

TOKEN_TYPE_ERROR = (-810, 'token is not unicode type.')

CLIENT_TYPE_ILLEGAL = (-820, 'client_type is illegal.')

CONTENT_TYPE_ERROR = (-830, 'content is not unicode type.')

TITLE_TYPE_ERROR = (-840, 'title is not unicode type.')

MSG_TYPE_ILLEGAL = (-850, 'msg_type is illegal.')

CUSTOM_INFO_TYPE_ERROR = (-860, 'custom_info is not dict type.')

ENVIRONMENT_ILLEGAL = (-870, 'environment is illegal.')