from .core import Device 

from .entity import (
    add,
    remove,
    device_tag,
    device_alias,
    device_regid,
)

__all__ = [
    Device,
    add,
    remove,
    device_tag,
    device_alias,
    device_regid,
]
